@@ -29,6 +29,7 @@ from reportlab.graphics.charts.piecharts import Pie
 from reportlab.graphics.charts.legends import Legend
 from reportlab.graphics.charts.spider import SpiderChart
 from reportlab.graphics.widgets.markers import makeMarker
+from reportlab.graphics.widgets.adjustableArrow import AdjustableArrowDrawing
 
 try:
     from reportlab.graphics import _renderPM
@@ -286,12 +287,20 @@ h1 = styleSheet['Heading1']
 h2 = styleSheet['Heading2']
 h3 = styleSheet['Heading3']
 
-def run_samples(S,kind='axes'):
+def eps():
+    try:
+        from rlextra.graphics import renderPS_SEP
+    except ImportError:
+        return []
+    else:
+        return ['eps']
+
+def run_samples(L):
     outDir = outputfile('charts-out')
-    for f in S:
-        if f.startswith('sample'):
-            d = S[f]()
-            d.save(formats=['pdf', 'gif', 'svg'],outDir=outDir, fnRoot='test_graphics_charts_%s_%s' % (kind,f))
+    for k,f,kind in L:
+        d = f()
+        if not isinstance(d,Drawing): continue
+        d.save(formats=['pdf', 'gif', 'svg', 'ps', 'py']+eps(),outDir=outDir, fnRoot='test_graphics_charts_%s_%s' % (kind,k))
 
 class ChartTestCase(unittest.TestCase):
     "Test chart classes."
@@ -447,6 +456,22 @@ class ChartTestCase(unittest.TestCase):
         drawing8 = sample8()
         story.append(drawing8)
         story.append(Spacer(0,1*cm))
+        story.append(Paragraph('The drawing is set as zero size, but the widget can still draw where you want it.', bt))
+        def makeArrow(scale=0.2,boxAnchor='c',**kwds):
+            A = AdjustableArrowDrawing()
+            A.width = A.height = 0
+            A._ZEROSIZE = True
+            A.adjustableArrow.scale = scale
+            A.adjustableArrow.boxAnchor = boxAnchor
+            for k,v in kwds.items():
+                setattr(A.adjustableArrow,k,v)
+            return A
+        A = makeArrow()
+        bb = A.getBounds()
+        deltax = 2 + max(bb[2]-bb[0],bb[3]-bb[1])
+        for i,angle in enumerate((0, 10, 20, 60, 90, 120, 180, 270, 315)):
+            story.append(makeArrow(y=-10,x=deltax*i,angle=angle,strokeColor=colors.black,strokeWidth=0.5,headSweep=-i*0.6))
+        story.append(Spacer(0,1*cm))
 
     @unittest.skipIf(not _renderPM,'no _renderPM')
     def test8(self):
@@ -973,7 +998,23 @@ class ChartTestCase(unittest.TestCase):
 
             return drawing
 
-        run_samples(locals())
+        def sample_hatching():
+            d = Drawing(width=110,height=110)
+            d.add(Hatching(spacings=(5,5),angles=(45,-45),xyLists=[5,5, 5,100, 99,105, 105,5],strokeWidth=1,strokeColor=colors.toColor('red'), strokeDashArray=None))
+            return d
+
+        def extract_samples():
+            S = [].extend
+            for m in ('barcharts','doughnut','linecharts','lineplots','piecharts','spider'):
+                mod = {}
+                exec('from reportlab.graphics.charts import %s as mod' % m ,mod)
+                mod  = mod['mod']
+                L = [(k,getattr(mod,k)) for k in dir(mod) if k.lower().startswith('sample')]
+                S([(k,v,m) for k,v in L if callable(v)])
+            return S.__self__
+
+        run_samples([(k,v,'axes') for k,v in locals().items() if k.lower().startswith('sample')])
+        run_samples(extract_samples())
 
     def test_legends(self):
         from reportlab.graphics.charts.legends import Legend, LineLegend, LineSwatch
@@ -1085,7 +1126,7 @@ class ChartTestCase(unittest.TestCase):
             d.legend.swatchCallout = LSwatchCallout(sw_names,'Helvetica',12)
             return d
 
-        run_samples(locals(),'legends')
+        run_samples([(k,v,'legends') for k,v in locals().items() if k.lower().startswith('sample')])
 
 def makeSuite():
     return makeSuiteForClasses(ChartTestCase)
